import { existsSync } from 'fs';
import { mkdir, rm, writeFile } from 'fs/promises';
import StyleDictionary from 'browser-style-dictionary/browser.js';
import { tsTypesDeclaration, tsFull, jsFull } from '../formats.mjs';
import { defu } from 'defu';
import { resolve } from 'pathe';
import chalk from 'chalk';
import { useLogger, requireModule } from '@nuxt/kit';

const name = "@nuxtjs/design-tokens-edge";
const version = "0.0.1-27636627.5c11574";

const MODULE_DEFAULTS = {
  tokens: true,
  server: false
};
const logger = useLogger("design-tokens");
chalk.magentaBright(name);
const resolveConfig = (layer, key, configFile = `${key}.config`) => {
  const value = layer.config?.designTokens?.[key] || MODULE_DEFAULTS[key];
  let config = {};
  let filePath;
  if (typeof value === "boolean") {
    filePath = resolve(layer.cwd, configFile);
  } else if (typeof value === "string") {
    filePath = resolve(layer.cwd, value);
  } else if (typeof value === "object") {
    config = value;
  }
  if (filePath) {
    try {
      const _file = requireModule(filePath, { clearCache: true });
      if (_file) {
        config = _file;
      }
    } catch (_) {
    }
  }
  return { filePath, config };
};
const resolveTokens = (layers) => {
  const tokensFilePaths = [];
  let tokens = {};
  const splitLayer = (layer) => {
    if (layer.config?.designTokens?.tokens || MODULE_DEFAULTS.tokens) {
      const { config: layerTokens, filePath: _layerTokensFilePath } = resolveConfig(layer, "tokens", "tokens.config");
      if (_layerTokensFilePath) {
        tokensFilePaths.push(_layerTokensFilePath);
      }
      tokens = defu(tokens, layerTokens);
    }
  };
  for (const layer of layers) {
    splitLayer(layer);
  }
  return { tokensFilePaths, tokens };
};
const createTokensDir = async (path) => {
  if (!existsSync(path)) {
    await mkdir(path, { recursive: true });
  }
};

const stubTokens = async (buildPath, force = false) => {
  const files = {
    "tokens.css": () => "/* This file is empty because no tokens has been provided. */",
    "tokens.scss": () => "/* This file is empty because no tokens has been provided. */",
    "tokens.json": () => "{}",
    "index.js": jsFull,
    "index.ts": tsFull,
    "types.d.ts": tsTypesDeclaration
  };
  for (const [file, stubbingFunction] of Object.entries(files)) {
    const path = `${buildPath}${file}`;
    if (force && existsSync(path)) {
      await rm(path);
    }
    if (!existsSync(path)) {
      await writeFile(path, stubbingFunction ? stubbingFunction({ tokens: {} }) : "");
    }
  }
};
const generateTokens = async (tokens, buildPath, silent = true) => {
  if (!existsSync(buildPath)) {
    await createTokensDir(buildPath);
  }
  if (!tokens || !Object.keys(tokens).length) {
    await stubTokens(buildPath, true);
    return;
  }
  let styleDictionary = StyleDictionary;
  styleDictionary.fileHeader = {};
  styleDictionary.registerTransformGroup({
    name: "tokens-js",
    transforms: ["name/cti/kebab", "size/px", "color/hex"]
  });
  styleDictionary.registerFormat({
    name: "typescript/types-declaration",
    formatter({ dictionary }) {
      return tsTypesDeclaration(dictionary);
    }
  });
  styleDictionary.registerFormat({
    name: "typescript/full",
    formatter({ dictionary }) {
      return tsFull(dictionary);
    }
  });
  styleDictionary.registerFormat({
    name: "javascript/full",
    formatter({ dictionary }) {
      return jsFull(dictionary);
    }
  });
  styleDictionary = await styleDictionary.extend({
    tokens,
    platforms: {
      scss: {
        transformGroup: "tokens-js",
        buildPath,
        files: [
          {
            destination: "tokens.scss",
            format: "scss/variables"
          }
        ]
      },
      json: {
        transformGroup: "tokens-js",
        buildPath,
        files: [
          {
            destination: "tokens.json",
            format: "json/flat"
          }
        ]
      },
      ts: {
        transformGroup: "tokens-js",
        buildPath,
        files: [
          {
            destination: "index.ts",
            format: "typescript/full"
          },
          {
            destination: "types.d.ts",
            format: "typescript/types-declaration"
          }
        ]
      },
      js: {
        transformGroup: "tokens-js",
        buildPath,
        files: [
          {
            destination: "index.js",
            format: "javascript/full"
          }
        ]
      },
      css: {
        transformGroup: "tokens-js",
        buildPath,
        files: [
          {
            destination: "tokens.css",
            format: "css/variables"
          }
        ]
      }
    }
  });
  if (silent) {
    console._log = console.log;
    console.log = () => {
    };
  }
  styleDictionary.cleanAllPlatforms();
  await new Promise((resolve) => setTimeout(resolve, 10));
  styleDictionary.buildAllPlatforms();
  await new Promise((resolve) => setTimeout(resolve, 10));
  if (silent) {
    console.log = console._log;
  }
};

export { MODULE_DEFAULTS as M, createTokensDir as c, generateTokens as g, logger as l, name as n, resolveTokens as r, stubTokens as s, version as v };
